// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file OpenMP.h */

#ifndef AMDIS_OPENMP_H
#define AMDIS_OPENMP_H

#ifdef _OPENMP
#include <omp.h>
#include <vector>
#endif

namespace AMDiS {

  using namespace std;
  
#ifdef _OPENMP

  template<typename T>
  class ThreadPrivate {
  public:
    ThreadPrivate()
      : data(omp_get_max_threads())
    {}

    ThreadPrivate(T val)
      : data(omp_get_max_threads(), val)
    {}

    inline T& get()
    {
#if (DEBUG != 0)
      if (omp_get_thread_num() >= data.size()) {
	cout << "Error in ThreadPrivate::get()!\n";
	exit(0);
      }
#endif
      return data[omp_get_thread_num()];
    }

    inline void set(T& val)
    {
#if (DEBUG != 0)
      if (omp_get_thread_num() >= data.size()) {
	cout << "Error in ThreadPrivate::set()!\n";
	exit(0);
      }
#endif
      data[omp_get_thread_num()] = val;
    }

  private:
    vector<T> data;
  };

#else

  template<typename T>
  class ThreadPrivate {
  public:
    ThreadPrivate() {}

    ThreadPrivate(T val)
      : data(val)
    {}

    inline T& get()
    {
      return data;
    }

    inline void set(T& val)
    {
      data = val;
    }

  private:
    T data;
  };

#endif

}

#endif
